package example.localplugin;

import gridsim.Gridlet;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Properties;

import schedframe.resources.PowerState;
import schedframe.scheduling.TaskInterface;
import schedframe.scheduling.events.SchedulingEvent;
import schedframe.scheduling.plugin.grid.ModuleList;
import test.rewolucja.GSSIMJobInterface;
import test.rewolucja.resources.manager.implementation.ClusterResourceManager;
import test.rewolucja.resources.manager.interfaces.ResourceManagerInterface;
import test.rewolucja.resources.physical.implementation.ComputingNode;
import test.rewolucja.scheduling.JobRegistryInterface;
import test.rewolucja.scheduling.plan.SchedulingPlanInterfaceNew;
import test.rewolucja.scheduling.plan.SchedulingPlanNew;
import test.rewolucja.scheduling.queue.Queue;
import test.rewolucja.scheduling.queue.QueueList;

public class FCFSConsolidationClusterLocalPlugin extends BaseLocalPlugin {
	
	private int scenario = 1;

	public FCFSConsolidationClusterLocalPlugin () {
	}

	public SchedulingPlanInterfaceNew schedule(SchedulingEvent event, QueueList queues, JobRegistryInterface jobRegistry,
			ResourceManagerInterface resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlanNew plan = new SchedulingPlanNew();
		// chose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
			// our tasks are placed only in first queue (see
			// BaseLocalPlugin.placeJobsInQueues() method)
			Queue q = queues.get(0);
			// check all tasks in queue

			for (int i = 0; i < q.size(); i++) {
				GSSIMJobInterface<?> job = q.get(i);
				TaskInterface<?> task = (TaskInterface<?>) job;
				// if status of the tasks in READY
				if (task.getStatus() == Gridlet.READY) {

					ComputingNode node = chooseResourcesForExecution(resourceManager, PowerState.ON, task);
					if (node  != null) {
						System.out.println("Uruchamiam na zasobie: " + node.getName());
						addToSchedulingPlan(plan, task, node.getName());
					} 
					else
					{
						node = chooseResourcesForExecution(resourceManager, PowerState.OFF, task);
						
						if( node != null)
						{
							System.out.println("Wlaczam wezel: " + node.getName());
							node.getPowerInterface().setPowerState( PowerState.ON);
							i--;
						}	
					}
				}
			}
			
			switch( scenario)
			{
				case 0: break;
				case 1:
					List<ComputingNode> nodes = resourceManager.getComputingNodes();
					for( ComputingNode node : nodes)
						if( node.getFreeProcessors().size() == node.getProcessorsNumber())
						{
							node.getPowerInterface().setPowerState( PowerState.OFF);
						}
					break;
			}
			

			break;
		}
		return plan;
	}
	
	private ComputingNode chooseResourcesForExecution(ClusterResourceManager resourceManager, PowerState status, TaskInterface<?> task) {

		List<ComputingNode> nodes = resourceManager.getComputingNodes();
		nodes = findSuitableNodes(task, status, nodes);
		
		Collections.sort(nodes, new Comparator<ComputingNode>(){
		    public int compare(ComputingNode node1, ComputingNode node2){    
		        return node1.getCategory().getName().compareTo(node2.getCategory().getName());
		    }
		});
		
		if(nodes.size() > 0)
			return nodes.get(0);
				 
		return null;
	}
	
	private List<ComputingNode> findSuitableNodes(TaskInterface<?> task, PowerState status, List<ComputingNode> nodes){
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 1;
		}
		
		List<ComputingNode> suitableNodes = new ArrayList<ComputingNode>();
		for(ComputingNode node: nodes)
		{
				switch( status)
				{
					case ON:
						if(node.getFreeProcessorsNumber() >= cpuRequest)
							suitableNodes.add(node);
						break;
					case OFF:
						if( node.getProcessorsNumber() >= cpuRequest)
							suitableNodes.add(node);
						break;
				}
		}
	
		return suitableNodes;
	}
	
	public String getName() {
		return getClass().getName();
	}

	public void init(Properties properties) {
		// no extra initialization is expected.
	}

}
